(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.2'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      7923,        218]*)
(*NotebookOutlinePosition[      8596,        241]*)
(*  CellTagsIndexPosition[      8552,        237]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[{
  StyleBox["Groebner Basis Methods for Multichannel Sampling with Unknown \
Offsets\n",
    FontSize->36],
  StyleBox["Luciano Sbaiz, Patrick Vandewalle, and Martin Vetterli",
    FontSize->18]
}], "Title"],

Cell["\<\
Code to reproduce Example 5.1 from the paper.
The paper, along with this code is available online at \
http://lcavwww.epfl.ch/reproducible_research/SbaizVV06 \
\>", "Abstract"],

Cell["\<\
Copyright (C) 2006 Laboratory of Audiovisual Communications \
(LCAV),Ecole Polytechnique Federale de Lausanne (EPFL),CH-1015 \
Lausanne,Switzerland

This program is free software;you can redistribute it and/or modify it under \
the terms of the GNU General Public License as published by the Free Software \
Foundation;either version 2 of the License,or (at your option) any later \
version.This software is distributed in the hope that it will be useful,but \
without any warranty;without even the implied warranty of merchantability or \
fitness for a particular purpose.See the GNU General Public License for more \
details (enclosed in the file GPL).\
\>", "Copyright"],

Cell["Latest modifications:June 13,2006", "Text"],

Cell["\<\
<<Statistics`ContinuousDistributions`
SeedRandom[1];
LL=5;
MM=2;
NN=4;
KK=(LL-1)/2;
ExactErrorList={};
EstimatedErrorList={};
For[simnr=1,simnr<251,simnr++,
{
y ={{y00,y01,y02,y03},{y10,y11,y12,y13}};
X ={X0,X1,X2,X3,X4};
z={z1};
vars =Join[X,z];
z=Prepend[z,1];
Xtrue = Table[0,{i,LL}];
Xtrue[[KK+1]]=Random[Integer,{-10,10}];
For[i=1,i<KK+1,
  i++,{Xtrue[[KK+1+i]]=Random[Integer,{-10,10}]+I*Random[Integer,{-10,10}];
    Xtrue[[KK+1-i]]=Conjugate[Xtrue[[KK+1+i]]];
    }];
ttrue = Table[0,{i,MM}];
For[i=2,i<MM+1,i++,ttrue[[i]]=Rational[Random[Integer,{1,100}],101]];
ztrue=Exp[I*2*\[Pi]*ttrue];
Plist=Table[0,{i,MM*NN}];Plistmatrix=Table[0,{i,MM*NN},{j,LL+1}];
For[n=1,n<NN+1,n++,
    For[m=1,m<MM+1,m++,{
          Plist[[n+NN*(m-1)]]=-y[[m]][[n]]z[[m]]^KK;
          Plistmatrix[[n+NN*(m-1),LL+1]]=y[[m]][[n]]z[[m]]^KK;
          For[l=1,l<LL+1,l++,{
              
              Plist[[n+NN*(m-1)]]=
                Plist[[n+NN*(m-1)]]+
                  Exp[I*2*\[Pi]*(n-1)(l-KK-1)/NN]z[[m]]^(l-1)*X[[l]];
              
              Plistmatrix[[n+NN*(m-1),l]]=
                Exp[I*2*\[Pi]*(n-1)(l-KK-1)/NN]z[[m]]^(l-1);
              }];
          }
        ];
    ];
reprules=Table[0,{i,LL+MM-1}];
For[l=1,l<LL+1,l++,reprules[[l]]=X[[l]]\[Rule]Xtrue[[l]]];
For[m=1,m<MM,m++,reprules[[LL+m]]=z[[m+1]]\[Rule]ztrue[[m+1]]];
test:=Plist /.reprules;
samples=Simplify[Solve[test\[Equal]0]];
realsamples=Re[y/.N[samples[[1]]]];
noise=Table[Random[NormalDistribution[0,1]],{i,MM},{j,NN}];
noisysamples=N[realsamples+noise];
newsamples=Table[0,{i,MM},{j,NN}];
noiselesssamples=Table[0,{i,MM},{j,NN}];
For[m=1,m<MM+1,m++,
    For[n=1,n<NN+1,n++,{
        noiselesssamples[[m,n]]=y[[m,n]]\[Rule]realsamples[[m,n]];
        newsamples[[m,n]]=y[[m,n]]\[Rule]noisysamples[[m,n]];
        }]];
noiselesssamples=Flatten[noiselesssamples];
samples=Flatten[newsamples];
polylist =Plist/.samples;
polylistnoiseless=Plist/.noiselesssamples;
polylistmatrix=Plistmatrix/.{samples};
coeff=Range[1,Length[polylist]];
coefflists=Subsets[coeff,{LL+MM-1}];
GroebnerSols=Table[0,{i,Length[coefflists]}];
For[i=1,i<Length[coefflists]+1,i++,
    GroebnerSols[[i]]=GroebnerBasis[polylist[[coefflists[[i]]]],vars]];
AllVarssolved=Table[0,{i,Length[GroebnerSols]}];
For [i=1,i<Length[GroebnerSols]+1,
    i++,{AllVarssolved[[i]]=Solve[GroebnerSols[[i]]\[Equal]0,vars];
      }];
AllVarssolved=Flatten[N[AllVarssolved],1];
zsolved={};
For[i=1,i<Length[GroebnerSols]+1,i++,{
    Temp=z/.AllVarssolved[[i]];
    zabs=Abs[Temp];
    onecount=Table[0,{i,MM-1}];
    For[k=1,k<MM,k++,{
        ztmp=1;
        onecount[[k]]=0;
        For[m=1,m<MM+1,m++,{
            If[zabs[[m]]>1+0.00001||zabs[[m]]<1-0.00001,ztmp=0];
            
            If[Abs[Re[Temp[[k,m]]]]<1+0.00001&&Abs[Re[Temp[[k,m]]]]>1-0.00001,\

              onecount[[k]]=onecount[[k]]+1];
            }];
        If[
          ztmp\[Equal]1&&onecount[[k]]\[NotEqual]MM&&
            Length[AllVarssolved[[i]]]==LL+MM-1,
          {If[Norm[polylist/.AllVarssolved[[i]]]<10,
              zsolved=Append[zsolved,AllVarssolved[[i]]]]}];
        }]
    }];
zVarsolved=z/.zsolved;
offsetsolved=Re[Log[zVarsolved]/(2\[Pi]*I)];
tSolved=Mean[offsetsolved];
zSolvedAvgRule=Table[0,{i,MM-1}];
For[i=2,i<MM+1,i++,
  zSolvedAvgRule[[i-1]]=z[[i]]\[Rule]N[Exp[I*2*\[Pi]*tSolved[[i]]]]];
CoeffMatrix=Take[polylistmatrix[[1]],{1,MM*NN},{1,LL}]/.zSolvedAvgRule;
SampleVector=Take[polylistmatrix[[1]],{1,MM*NN},{LL+1,LL+1}]/.zSolvedAvgRule;
Xcomputed=PseudoInverse[CoeffMatrix].SampleVector;
XcomputedRule=Table[0,{i,LL}];
For[i=1,i<LL+1,i++,XcomputedRule[[i]]=X[[i]]\[Rule]Xcomputed[[i]]];
TfromZ=N[Log[z/.zSolvedAvgRule]/(I*2*\[Pi])];
For[j=1,j<MM+1,j++,
    If[Im[TfromZ[[j]]]<0.0000000001,TfromZ[[j]]=Re[TfromZ[[j]]];]];
CoeffMatrixExactOffsets=
    Take[polylistmatrix[[1]],{1,MM*NN},{1,LL}]/.Take[reprules,-(MM-1)];
SampleVectorExactOffsets=
    Take[polylistmatrix[[1]],{1,MM*NN},{LL+1,LL+1}]/.Take[reprules,-(MM-1)];
XcomputedExactOffsets=
    PseudoInverse[CoeffMatrixExactOffsets].SampleVectorExactOffsets;
Print[\"Solution: \"];
Print[\"Exact offsets:\",N[ttrue],\"\\n\",
    \"Estimated offsets:\",TfromZ];
Print[\"Exact coefficients:\",Xtrue,\"\\n\\n\",
    \"Estimated coefficients with exact offsets:\",
    Flatten[XcomputedExactOffsets],\"\\n\",
    \"Coefficient error using exact shifts: \",
    Norm[XcomputedExactOffsets-Xtrue],\"\\n\\n\",
    \"Estimated coefficients with estimated shifts:\",Xcomputed,\"\\n\",
    \"Coefficient error using estimated shifts: \",
    Norm[Flatten[Xcomputed]-Xtrue]];
ExactErrorList=Append[ExactErrorList,Norm[XcomputedExactOffsets-Xtrue]/Norm[\
Xtrue]];
EstimatedErrorList=Append[EstimatedErrorList,Norm[Flatten[Xcomputed]-Xtrue]/\
Norm[Xtrue]];
ClearAll[X,X0,X1,X2,X3,X4,y,y00,y01,y02,y03,y10,y11,y12,y13,z,z1,Xtrue,ttrue,\
ztrue,Plist,Plistmatrix,reprules,test];
}];
Print[ExactErrorList,\"; \",Mean[ExactErrorList]];
Print[EstimatedErrorList,\"; \",Mean[EstimatedErrorList]];\
\>", "Input",
  CellLabel->"In[12]:="]
},
FrontEndVersion->"5.2 for Macintosh",
ScreenRectangle->{{0, 1280}, {0, 793}},
WindowSize->{793, 771},
WindowMargins->{{Automatic, 124}, {Automatic, 0}},
StyleDefinitions -> "Classroom.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 222, 6, 144, "Title"],
Cell[1979, 59, 186, 4, 36, "Abstract"],
Cell[2168, 65, 683, 12, 102, "Copyright"],
Cell[2854, 79, 49, 0, 26, "Text"],
Cell[2906, 81, 5013, 135, 1997, "Input"]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

